<?php
/*
 * @version $Id: config.class.php 73 2012-07-09 16:03:30Z remi $
 -------------------------------------------------------------------------
 loadentity - LoadEntity plugin for GLPI
 Copyright (C) 2003-2011 by the loadentity Development Team.

 https://forge.indepnet.net/projects/loadentity
 -------------------------------------------------------------------------

 LICENSE

 This file is part of loadentity.

 loadentity is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 loadentity is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with loadentity. If not, see <http://www.gnu.org/licenses/>.
 --------------------------------------------------------------------------
 */

// Original Author of file: Remi Collet
// Purpose of file:
// ----------------------------------------------------------------------


if (!defined("PLUGIN_LOADENTITY_UPLOAD_DIR")) {
   define  ("PLUGIN_LOADENTITY_UPLOAD_DIR", GLPI_PLUGIN_DOC_DIR."/loadentity/");
}


class PluginLoadentityConfig extends CommonDBTM{

   // GLPI DB Schema : 1.3 is for 0.83
   const DB_SCHEMA                = '1.3';

   private $baseurl               = "plugins/dumpentity/front";
   protected $displaylist         = false;
   public $auto_message_on_action = false;


   function canCreate() {
      return Session::haveRight('config', 'w');
   }


   function canView() {
      return Session::haveRight('config', 'r');
   }


   function canDelete() {
      return false;
   }


   function showStatus() {
      global $LANG,$DB;

      echo "<table class='tab_cadre' cellpadding='5'>";
      echo "<tr><th colspan='3'>".$LANG['plugin_loadentity']['setup'][6]."</th></tr>";

      $response = $this->PreliminaryNegociation();

      if (!$response) {
         echo "<tr class='tab_bg_1'><td class='center b' colspan='3'>" .
                $LANG['plugin_loadentity']['setup'][11]."</td></tr>";
         echo "</table>";
         return false;
      }

      echo "<tr class='tab_bg_4'><td colspan='3' class='center'>" .
             $LANG['plugin_loadentity']['setup'][9]. "&nbsp;:&nbsp;";
      if (empty($this->fields["user"])) {
         echo $LANG['plugin_loadentity']['setup'][10]. "</td>";
      } else {
         echo Html::convDateTime($this->fields["date_sync"])." (".$this->fields["user"]. ")</td></tr>";
      }

      $tables = $this->getTables();
      if (count($tables)) {
         echo "<tr class='tab_bg_5'><".
              "td class='center' colspan='3'>".$LANG['plugin_loadentity']['setup'][8]."</td></tr>";
         $i = 0;
         foreach ($tables as $nom => $desc) {
            if ($i%3==0) {
               echo "<tr class='tab_bg_1'>";
            }
            echo "<td>". $desc . "</td>";

            if ($i%3==2) {
               echo "</tr>\n";
            }
            $i++;
         }

         if ($i%3==1) {
            echo "<td>&nbsp;</td><td>&nbsp;</td></tr>\n";
         }
         if ($i%3==2) {
            echo "<td>&nbsp;</td></tr>\n";
         }

      } else {
         echo "<tr class='tab_bg_4'>".
              "<td class='center b' colspan='3'>".$LANG['plugin_loadentity']['setup'][7]."</td>".
              "</tr>";
      }
      echo "</table>";
      return true;
   }


   function defineTabs($options=array()) {
      global $LANG;

      $ong = array();
      $this->addStandardTab(__CLASS__, $ong, $options);
      return $ong;
   }


   static function getTypeName() {
      global $LANG;

      return $LANG['plugin_loadentity']['setup'][1];
   }


   function showForm($ID, $options=array()) {
      global $LANG,$DB;

      if ($ID > 0) {
         $this->check($ID,'r');
      } else {
         $this->check(-1,'w');
         $this->getEmpty();
      }

      $options['colspan'] = 1;
      $this->showTabs($options);
      $this->showFormHeader($options);

      //echo "<tr><th colspan='2' align='center'>".$LANG['plugin_loadentity']['setup'][2]."</th></tr>";

      echo "<tr class='tab_bg_1'>";
      echo "<td>" .$LANG['plugin_loadentity']['setup'][5]. "&nbsp;:&nbsp;</td>";
      echo "<td><input name='url' size='40' value='". ($ID>0 ? $this->fields["url"]:"http://")."'/>";
      echo "</td></tr>\n";

      $this->showFormButtons($options);
      echo "<div id='tabcontent'></div>";
      echo "<script type='text/javascript'>loadDefaultTab();</script>";
   }


   function getDumpURL() {
      return $this->baseurl.'/getcsv.php';
   }


   function getChecksURL() {
      return $this->fields["url"].$this->baseurl.'/checks.php';
   }


   function getTabNameForItem(CommonGLPI $item, $withtemplate=0) {
      global $LANG;

      if ($item->getType() == __CLASS__) {
         if ($item->getField('id')) {
            return array(1 => $LANG['common'][12]);
         }
      }
      return '';
   }


   static function displayTabContentForItem(CommonGLPI $item, $tabnum=1, $withtemplate=0) {

      if ($item->getType() == __CLASS__) {
         $item->showStatus();
      }
      return true;
   }

   function PreliminaryNegociation() {

      $plugin_infos = plugin_version_loadentity();
      $buf          = @file_get_contents($this->getChecksURL()."?check=1&version=".
                                         urlencode($plugin_infos["version"]).
                                         "&dbschema=".self::DB_SCHEMA,"rb");

      return ($buf === '0');
   }

   function getTables () {

      $tables = array();

      $fic = fopen($this->fields["url"].$this->getDumpURL(),"r");
      if ($fic) {
         // Header
         $data = fgetcsv($fic,1000,';','"');
         if ($data[0] == "name" && $data[1] == "description") {
            for ($i=0 ; $data=fgetcsv($fic,1000,';','"') ; $i++) {
               $tables[$data[0]] = $data[1];
            }
         }
         fclose($fic);
      }
      asort($tables);
      return $tables;
   }


   function isTimeToDump () {

      $buf = @file_get_contents($this->getChecksURL()."?check_time=1","r");
      return ($buf == 1);
   }


   function loadTable ($table, $recursive=false) {
      global $DB;

      if ($table == "glpi_users") {
         $Sql = "DELETE FROM `glpi_users`
                 WHERE `name` != 'glpi'";
      } else {
         $Sql = "TRUNCATE `$table`";
      }
      if (!$DB->query($Sql)) {
         return 0;
      }

      $Sql = "LOAD DATA LOCAL INFILE '".PLUGIN_LOADENTITY_UPLOAD_DIR.$table.".csv' " .
             "INTO TABLE `$table` ";

      if (mysql_get_server_info() >= "5.0.38") {
         $Sql .= "CHARACTER SET 'utf8' ";
      }

      $Sql .= "FIELDS TERMINATED BY ';'
               OPTIONALLY ENCLOSED BY '\"'
               LINES TERMINATED BY '\r\n'
               IGNORE 1 LINES;";
      $res = $DB->query($Sql);
      if ($res) {
         $nb = $DB->affected_rows($res);
      } else {
         return -1;
      }

      if ($table == "glpi_users_profiles") {
         // Change dynamic to static profile as no "rules" loaded
         $Sql = "UPDATE `$table`
                 SET `dynamic` = '0'";
         if (!$recursive)
         $Sql .= ", entities_id = '0'";

         $DB->query($Sql);

      } else if (!$recursive && FieldExists($table, "entities_id")) {
         $Sql = "UPDATE `$table`
                 SET `entities_id` = '0'";
         $DB->query($Sql);
      }
      @unlink(PLUGIN_LOADENTITY_UPLOAD_DIR.$table.".csv");

      return $nb;
   }


   function isDumpRecursive() {

      $buf = @file_get_contents($this->getChecksURL()."?recursive","rb");
      return ($buf == 1);
   }


   function loadFile($table) {

      $dst = @fopen(PLUGIN_LOADENTITY_UPLOAD_DIR.$table.".csv", "wb");
      if ($dst === false) {
         return -1;
      }

      ini_set("default_socket_timeout", "300"); // For very big table

      $rd = 0;
      $wr = 0;
      if (function_exists("gzopen")) {
         $fic = gzopen($this->fields["url"].$this->getDumpURL()."?table=$table&gzip=1","rb");
         if ($fic) {
            while ($buf=gzread($fic,4096)) {
               $rd += strlen($buf);
               $wr += fwrite($dst, $buf);
            }
            gzclose($fic);
         } else {
            fclose($dst);
            return -1;
         }

      } else {
         $fic = fopen($this->fields["url"].$this->getDumpURL()."?table=$table","rb");
         if ($fic) {
            while ($buf=fread($fic,4096)) {
               $rd += strlen($buf);
               $wr += fwrite($dst, $buf);
            }
            fclose ($fic);
         } else {
            fclose($dst);
            return -1;
         }
      }
      fclose ($dst);
      return ($rd==$wr ? $wr : -1);
   }


   function loadText (&$tables, $log, $recursive) {
      global $LANG,$DB;

      $tables = $this->getTables();
      if (!count($tables)) {
         return false;
      }

      fwrite($log, "Downloading files\n");
      foreach ($tables as $table => $descr) {
         $len[$table] = $this->loadFile($table);
         if ($len >= 0) {
            fwrite($log, "\t$descr downloaded ".$len[$table]." byte(s)\n");
         } else {
            fwrite($log, "\t$descr not downloaded\n");
            return false;
         }
      }
      fwrite($log,"Loading tables\n");
      foreach ($tables as $table => $descr) {
         $nb = $this->loadTable($table, $recursive);
         if ($nb >= 0) {
            fwrite($log, "\t$descr loaded $nb rows\n");
         } else {
            fwrite($log, "\t$descr not loaded\n");
            return false;
         }
      }
      $this->update(array('id'        => 1,
                          'user'      => 'cron',
                          'date_sync' => date("Y-m-d H:i:s")));
      return true;
   }


   function loadHtml (&$tables,$recursive) {
      global $LANG,$DB;

      print <<<END
      <script type='text/javascript'>
      function ImgStatus(Obj,Img) {
         var x;

         if ((x=document.getElementById(Obj)) != null) {
            x.src = Img;
         }
      }
      function TextStatus(Id) {
         var x;

         if ((x=document.getElementById(8000)) != null) {
            x.style.display = 'none';
         }
         if ((x=document.getElementById(Id)) != null) {
            x.style.display = 'inline';
         }
      }
      function ValueStatus(Id, Val) {
         var x;

         if ((x=document.getElementById(Id)) != null) {
            x.innerHTML = Val;
         }
      }
      </script>
END;
      echo "\n";

      echo "<table class='tab_cadre' cellpadding='6'>\n";
      echo "<tr class='tab_bg_1'><th colspan='4'>". $LANG['plugin_loadentity']['load'][1]."</td/></tr>\n";

      echo "<tr class='tab_bg_1'><td>".$LANG['plugin_loadentity']['load'][3]."</td>".
            "<td>".$LANG['plugin_loadentity']['load'][4]."</td>".
            "<td class='right'>".$LANG['plugin_loadentity']['load'][5]."</td>".
            "<td class='right'>".$LANG['plugin_loadentity']['load'][6]."</td></tr>\n";

      $tables = $this->getTables();
      $i = 0;
      foreach ($tables as $table => $descr) {
         echo "<tr class='tab_bg_1'><td>$descr</td><td>$table</td>" .
               "<td class='right'><span id='" . ($i+3000) . "'>&nbsp;</span>&nbsp;".
                  "<img id='" . ($i+4000) . "' src='" . GLPI_ROOT . "/pics/rdv.png'></td>".
               "<td class='right'><span id='" . ($i+5000) . "'>&nbsp;</span>&nbsp;".
                  "<img id='" . ($i+6000) . "' src='" . GLPI_ROOT . "/pics/rdv.png'></td></tr>\n";
         $i++;
      }

      echo "<tr class='tab_bg_4'><td class='center' colspan='4'>".
            "<div id='8000'>". $LANG['plugin_loadentity']['load'][8]."</div>".
            "<div id='8001' style='display:none'>". $LANG['plugin_loadentity']['load'][9]."</div>".
            "<div id='8002' style='display:none'><strong>". $LANG['plugin_loadentity']['load'][10].
               "</strong></div></td/></tr>\n";
      echo "</table>";

      if (!count($tables)) {
         return false;
      }

      $i = 4000;
      foreach ($tables as $table => $descr) {
         echo "<script type='text/javascript'>ImgStatus($i,'".GLPI_ROOT."/pics/actualiser.png');".
              "</script>\n";
         Html::glpi_flush();

         $len[$table] = $this->loadFile($table);
         if ($len[$table] >= 0) {
            echo "<script type='text/javascript'>ValueStatus(".($i-1000).",".$len[$table].");".
                  "ImgStatus($i,'".GLPI_ROOT."/pics/greenbutton.png');</script>\n";
         } else {
            echo "<script type='text/javascript'>TextStatus(8002);".
                  "ImgStatus($i,'".GLPI_ROOT."/pics/redbutton.png');</script>\n";
            return false;
         }
         $i++;
      }
      $i = 6000;
      foreach ($tables as $table => $descr) {
         echo "<script type='text/javascript'>ImgStatus($i,'".GLPI_ROOT."/pics/actualiser.png');".
              "</script>\n";
         Html::glpi_flush();

         $nb = $this->loadTable($table, $recursive);
         if ($nb >= 0) {
            echo "<script type='text/javascript'>ValueStatus(".($i-1000).",$nb);".
                     "ImgStatus($i,'".GLPI_ROOT."/pics/greenbutton.png');</script>\n";
         } else {
            echo "<script type='text/javascript'>TextStatus(8002);".
                     "ImgStatus($i,'".GLPI_ROOT."/pics/redbutton.png');</script>\n";
            return false;
         }
         $i++;
      }
      echo "<script type='text/javascript'>TextStatus(8001);</script>\n";

      $this->update(array('id'        => 1,
                          'user'      => $_SESSION["glpiname"],
                          'date_sync' => $_SESSION["glpi_currenttime"]));

      return true;
   }


   function load ($log=false) {

      $recursive = $this->isDumpRecursive();
      $tables    = $this->getTables();
      if (!count($tables)) {
         return false;
      }

      if ($log) {
         $ret = $this->loadText($tables, $log, $recursive);
      } else {
         $ret = $this->loadHtml($tables, $recursive);
      }

      return $ret;
   }

   function prepareInputForAdd($input) {
      if (isset($input['url']) && $input['url'] && substr($input['url'],-1) != '/') {
         $input['url'] .= '/';
      }
      return $input;
   }

   function prepareInputForUpdate($input) {
      return $this->prepareInputForAdd($input);
   }
}
?>
